import 'package:flutter/material.dart';
import 'package:superellipse_shape/superellipse_shape.dart';

import '../mcg_extension.dart';

class AlertAction {
  final String? text;
  final Function? onPressed;
  AlertAction({this.text, this.onPressed});
}

enum AlertType {
  successful,
  info,
  warning,
  danger,
}

class Alert {
  Alert._();

  static const Color _sucColor = Color(0xff65BA6B);
  static const Color _sucBorder = Color(0xffC4E3C9);
  static const Color _infoColor = Color(0xff2D6ADD);
  static const Color _infoBorder = Color(0xffB2CCEE);
  static const Color _warnColor = Color(0xffE19936);
  static const Color _warnBorder = Color(0xffF4DEB4);
  static const Color _dangerColor = Color(0xffDA5A39);
  static const Color _dangerBorder = Color(0xffEDC7BD);

  static void saveErr() => show(message: 'saveerr'.translate, type: AlertType.danger);
  static void deleteErr() => show(type: AlertType.danger, message: 'deleteerr'.translate);
  static void saveSuc() => show(message: 'savesuc'.translate, type: AlertType.successful);
  static void deleteSuc() => show(message: 'deletesuc'.translate, type: AlertType.successful);
  static void fillRequired() => show(message: 'fillrequired'.translate, type: AlertType.warning);
  static void nothingFoundToSave() => show(message: 'nothingfoundtosave'.translate, type: AlertType.warning);
  static void anError() => show(message: 'anerror'.translate, type: AlertType.danger);

//	static void get saveSuc => Get.

  static void alert({String? title, required String message, AlertType type = AlertType.info}) => show(title: title, message: message, type: type);

  static const double _maxAlertWidth = 450;
  static void show({AlertType type = AlertType.info, String? title, required String message, bool autoClose = true}) {
    final _isTablet = (Get.context?.screenWidth ?? 0) > 466 && (Get.context?.screenHeight ?? 0) > 500;
    var _leftInsets = 8.0;
    var _bottomInsets = 8.0;
    if (_isTablet) {
      _bottomInsets = 16;
      _leftInsets = (Get.context?.screenWidth ?? 0) - 458;
    }
    Get.rawSnackbar(
      maxWidth: _maxAlertWidth,
      snackPosition: _isTablet ? SnackPosition.BOTTOM : SnackPosition.BOTTOM,
      margin: EdgeInsets.only(left: _leftInsets, bottom: _bottomInsets, top: 8, right: 8),
      padding: const EdgeInsets.only(left: 8, bottom: 8, top: 8, right: 8),
      backgroundColor: type == AlertType.successful
          ? _sucBorder.withAlpha(150)
          : type == AlertType.info
              ? _infoBorder.withAlpha(150)
              : type == AlertType.warning
                  ? _warnBorder.withAlpha(150)
                  : _dangerBorder.withAlpha(150),

      borderColor: type == AlertType.successful
          ? _sucBorder
          : type == AlertType.info
              ? _infoBorder
              : type == AlertType.warning
                  ? _warnBorder
                  : _dangerBorder,
      barBlur: 6,
      messageText: Row(
        children: [
          Container(
            width: 48,
            height: 48,
            alignment: Alignment.center,
            decoration: ShapeDecoration(
                shape: SuperellipseShape(borderRadius: BorderRadius.circular(44)),
                color: type == AlertType.successful
                    ? _sucColor
                    : type == AlertType.info
                        ? _infoColor
                        : type == AlertType.warning
                            ? _warnColor
                            : _dangerColor),
            child: Icon(
              type == AlertType.successful
                  ? Icons.check_circle_rounded
                  : type == AlertType.info
                      ? Icons.info_rounded
                      : type == AlertType.warning
                          ? Icons.warning_rounded
                          : Icons.dangerous_rounded,
              color: Colors.white,
              size: 36,
            ),
          ),
          16.width,
          Expanded(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                if (title != null) title.text.bold.color(Colors.black).fontSize(20).make(),
                message.text.color(Colors.black).fontSize(20).make(),
              ],
            ),
          ),
          Icons.clear_rounded.icon
              .onPressed(() {
                if (Get.isSnackbarOpen == true) Get.back();
              })
              .color(const Color(0xff343C45))
              .make()
        ],
      ),
      //   titleText: const SizedBox(),

      snackStyle: SnackStyle.FLOATING,
      borderRadius: 16,

      animationDuration: 444.milliseconds,
      isDismissible: autoClose,
      duration: autoClose ? const Duration(seconds: 3) : const Duration(seconds: 3000),
    );
  }

  static Future<dynamic> advance({String? title, required String message, Color? bgColor, List<AlertAction>? rowActions, List<AlertAction>? columnActions, bool closeButton = false, bool danger = false, IconData? iconData, String? assetImage}) async {
    var autoClose = !closeButton && rowActions == null && columnActions == null;

    Get.snackbar(title ?? '', '',
        borderRadius: 16,
        snackPosition: SnackPosition.TOP,
        snackStyle: SnackStyle.FLOATING,
        margin: const EdgeInsets.all(16),
        borderColor: Colors.white.withAlpha(50),
        padding: EdgeInsets.only(left: 16, right: 16, top: 16, bottom: rowActions == null ? 16 : 1),
        borderWidth: 1,
        isDismissible: autoClose,
        animationDuration: const Duration(milliseconds: 500),
        backgroundColor: (bgColor ?? (danger ? Colors.redAccent.withAlpha(230) : (columnActions != null ? const Color(0xff3C3E3F) : Colors.blueAccent))),
        duration: autoClose ? const Duration(seconds: 3) : const Duration(seconds: 3000),
        colorText: Colors.white,
        titleText: title.safeLength < 1 ? const SizedBox() : null,
        messageText: _DetailWidget(
            dismiss: Get.back,
            message: message,
            danger: danger,
            bgColor: bgColor,
            rowActions: rowActions,
            closeButton: closeButton,
            columnActions: columnActions,
            iconWidget: iconData != null ? Icon(iconData, color: Colors.white) : (assetImage != null ? Image.asset(assetImage, width: 70) : const Icon(Icons.info, color: Colors.white))));
  }
}

class _DetailWidget extends StatelessWidget {
  final String? message;
  final bool? danger;
  final Function()? dismiss;
  final Widget? iconWidget;

  final List<AlertAction>? rowActions;
  final List<AlertAction>? columnActions;
  final bool closeButton;
  final Color? bgColor;

  _DetailWidget({this.message, this.danger, this.bgColor, this.dismiss, this.rowActions, this.columnActions, this.closeButton = false, this.iconWidget});

  @override
  Widget build(BuildContext context) {
    return SizedBox(
        width: double.infinity,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Row(
              children: <Widget>[iconWidget!, const SizedBox(width: 16), Expanded(child: message.text.color(Colors.white).fontSize(16).bold.make()), if (closeButton) IconButton(icon: const Icon(Icons.cancel, color: Colors.white), onPressed: dismiss)],
            ),
            if (rowActions != null)
              Row(
                mainAxisAlignment: MainAxisAlignment.end,
                children: <Widget>[
                  ...rowActions!
                      .map((item) => TextButton(
                            child: item.text.text.color(Colors.white).make(),
                            onPressed: () {
                              dismiss!();
                              item.onPressed!();
                            },
                          ))
                      .toList()
                ],
              ),
            if (columnActions != null)
              Column(
                children: <Widget>[
                  ...columnActions!.map((item) {
                    return Column(
                      children: <Widget>[
                        Container(
                          color: Colors.white.withAlpha(50),
                          width: double.infinity,
                          height: 1,
                        ),
                        TextButton(
                            onPressed: () {
                              dismiss!();
                              item.onPressed!();
                            },
                            child: Container(
                              alignment: Alignment.center,
                              padding: const EdgeInsets.symmetric(vertical: 10, horizontal: 16),
                              child: item.text.text.color(Colors.white).make(),
                            )),
                      ],
                    );
                  }).toList()
                ],
              )
          ],
        ));
  }
}
